require(tidyverse)
require(bmbstats)
require(lme4)

set.seed(1667)

# Generate data
n_athletes <- 10
n_trials <- 3
n_sim <- 100
intercept_min <- 80
intercept_max <- 120
slope_min <- -5
slope_max <- 8
random_min <- 5
random_max <- 15

df <- expand_grid(
  simulation = seq(1, n_sim),
  athlete = LETTERS[seq(1, n_athletes)]
) %>%
  group_by(simulation) %>%
  mutate(
    intercept = runif(n(), intercept_min, intercept_max),
    slope = runif(n(), slope_min, slope_max),
    random = runif(n(), random_min, random_max)
  ) %>%
  ungroup() %>%
  expand_grid(
    trial = seq(0, n_trials - 1)
  ) %>%
  mutate(
    true_value = intercept + slope * trial,
    random_error = rnorm(n(), 0, random),
    observed_value = true_value + random_error
  )

# Plot
one_sim_data <- filter(df, simulation == 1)

gg <- ggplot(
  one_sim_data,
  aes(x = trial, y = true_value, group = athlete)
) +
  theme_minimal() +
  geom_line(alpha = 0.5) +
  geom_point(aes(y = observed_value), alpha = 0.5)

gg

gg + facet_wrap(~athlete)

# ============================
# Check RSE between two trials

# Convert to wide format
one_sim_data_wide <- one_sim_data %>%
  pivot_longer(cols = -(1:6)) %>%
  pivot_wider(
    values_from = "value",
    names_from = c("name", "trial")
  )

# ---------------------------
# Method of the differences
plot_pair_BA(
  predictor = one_sim_data_wide$observed_value_0,
  predictor_label = "Trial 0",
  outcome = one_sim_data_wide$observed_value_1,
  outcome_label = "Trial 1",
  SESOI_lower = -0.2 * sd_pooled(one_sim_data_wide$observed_value_0, one_sim_data_wide$observed_value_1),
  SESOI_upper = 0.2 * sd_pooled(one_sim_data_wide$observed_value_0, one_sim_data_wide$observed_value_1),
  na.rm = TRUE
)

sd(one_sim_data_wide$observed_value_1 - one_sim_data_wide$observed_value_0) / sqrt(2)

# Linear regression
plot_pair_lm(
  predictor = one_sim_data_wide$observed_value_0,
  predictor_label = "Trial 0",
  outcome = one_sim_data_wide$observed_value_1,
  outcome_label = "Trial 1",
  SESOI_lower = -0.2 * sd_pooled(one_sim_data_wide$observed_value_0, one_sim_data_wide$observed_value_1),
  SESOI_upper = 0.2 * sd_pooled(one_sim_data_wide$observed_value_0, one_sim_data_wide$observed_value_1),
  na.rm = TRUE
)

m1 <- lm(
  observed_value_1 ~ observed_value_0,
  one_sim_data_wide
)

summary(m1)$sigma / sqrt(2)

# Ordinary least products
plot_pair_OLP(
  predictor = one_sim_data_wide$observed_value_0,
  predictor_label = "Trial 0",
  outcome = one_sim_data_wide$observed_value_1,
  outcome_label = "Trial 1",
  SESOI_lower = -0.2 * sd_pooled(one_sim_data_wide$observed_value_0, one_sim_data_wide$observed_value_1),
  SESOI_upper = 0.2 * sd_pooled(one_sim_data_wide$observed_value_0, one_sim_data_wide$observed_value_1),
  na.rm = TRUE
)

OLP_regression(
  predictor = one_sim_data_wide$observed_value_0,
  outcome = one_sim_data_wide$observed_value_1
)$rse / sqrt(2)

# ===========================
# Mixed models

# Function for plotting
plot_model <- function(model, data) {
  data$pred_obs <- predict(model, data)

  ggplot(data, aes(x = trial)) +
    theme_minimal() +
    geom_line(aes(y = true_value), linetype = "dashed", alpha = 0.5) +
    geom_point(aes(y = observed_value), alpha = 0.5) +
    geom_line(aes(y = pred_obs), color = "red") +
    facet_wrap(~athlete) +
    ylab(NULL)
}

# Individual models
mm0 <- lmList(
  observed_value ~ trial | athlete,
  one_sim_data
)

summary(mm0)
summary(mm0)$RSE

# Plot
plot_model(mm0, one_sim_data)

# Random intercept
mm1 <- lmer(
  observed_value ~ trial + (1 | athlete),
  one_sim_data
)

summary(mm1)

summary(mm1)$sigma

# Plot
plot_model(mm1, one_sim_data)

# Random slope and intercept
mm2 <- lmer(
  observed_value ~ trial + (1 + trial | athlete),
  one_sim_data
)

summary(mm2)

# Plot
plot_model(mm2, one_sim_data)


# Ordinal intervention levels
one_sim_data$trial_nom <- factor(one_sim_data$trial)

mm3 <- lmer(
  observed_value ~ trial_nom + (1 | athlete),
  one_sim_data
)

summary(mm3)

# Plot
plot_model(mm3, one_sim_data)

# ===========================
# Final table TE

get_TE <- function(data) {
  data_wide <- data %>%
    pivot_longer(cols = -(1:6)) %>%
    pivot_wider(
      values_from = "value",
      names_from = c("name", "trial")
    )

  TE_diff <- sd(data_wide$observed_value_1 - data_wide$observed_value_0) / sqrt(2)

  TE_lm <- summary(
    lm(observed_value_1 ~ observed_value_0, data_wide)
  )$sigma / sqrt(2)

  TE_olp <- OLP_regression(
    predictor = data_wide$observed_value_0,
    outcome = data_wide$observed_value_1
  )$rse / sqrt(2)

  
  mm0 <- lmList(
    observed_value ~ trial | athlete,
    data
  )
  
  mm1 <- lmer(
    observed_value ~ trial + (1 | athlete),
    data
  )

  mm2 <- lmer(
    observed_value ~ trial + (1 + trial | athlete),
    data
  )

  data$trial_nom <- factor(data$trial)

  mm3 <- lmer(
    observed_value ~ trial_nom + (1 | athlete),
    data
  )

  df <- tribble(
    ~model, ~TE,
    "Diff", TE_diff,
    "LM", TE_lm,
    "OLP", TE_olp,
    "Rand individual", summary(mm0)$RSE,
    "Rand int", summary(mm1)$sigma,
    "Rand int+slope", summary(mm2)$sigma,
    "Rant int (nom)", summary(mm3)$sigma
  )

  return(df)
}

# ===============================
TE <- df %>%
  group_by(simulation) %>%
  do(get_TE(.))

ggplot(TE, aes(y = TE, x = model)) +
  theme_minimal() +
  geom_boxplot()
